@@ -1,18 +1,22 @@
 import asyncio
 import socket
-import typing
-from asyncio import BaseTransport, StreamWriter
-from typing import Iterable
+from ssl import SSLContext
+from typing import Any, Iterable, NamedTuple, Optional, List, Tuple
 
-from aiohttp import TCPConnector, ClientConnectorError
-from aiohttp.abc import AbstractResolver
+from aiohttp import ClientConnectorError, TCPConnector
+from aiohttp.abc import AbstractResolver, ResolveResult
 from aiohttp.client_proto import ResponseHandler
 from python_socks import ProxyType, parse_proxy_url
 from python_socks.async_.asyncio.v2 import Proxy
 
 
 class NoResolver(AbstractResolver):
-    async def resolve(self, host, port=0, family=socket.AF_INET):
+    async def resolve(
+        self,
+        host: str,
+        port: int = 0,
+        family: socket.AddressFamily = socket.AF_INET,  # pylint: disable=no-member
+    ) -> List[ResolveResult]:
         return [
             {
                 'hostname': host,
@@ -36,23 +40,58 @@ class _ResponseHandler(ResponseHandler):
     See StreamWriter.__del__ method (was added in Python 3.11.5)
     """
 
-    def __init__(self, loop: asyncio.AbstractEventLoop, writer: StreamWriter):
+    def __init__(self, loop: asyncio.AbstractEventLoop, writer: asyncio.StreamWriter) -> None:
         super().__init__(loop)
         self._writer = writer
 
 
-class ProxyConnector(TCPConnector):
+class _BaseProxyConnector(TCPConnector):
+    async def _wrap_create_connection(
+        self,
+        *args,
+        addr_infos,
+        req,
+        timeout,
+        client_error=ClientConnectorError,
+        **kwargs,
+    ) -> Tuple[asyncio.Transport, ResponseHandler]:
+        try:
+            host: str = addr_infos[0][4][0]
+            port: int = addr_infos[0][4][1]
+        except IndexError as e:  # pragma: no cover
+            raise ValueError('Invalid arg: `addr_infos`') from e
+
+        ssl: Optional[SSLContext] = kwargs.get('ssl')  # type: ignore
+
+        return await self._connect_via_proxy(
+            host=host,
+            port=port,
+            ssl=ssl,
+            timeout=timeout.sock_connect,
+        )
+
+    async def _connect_via_proxy(
+        self,
+        host: str,
+        port: int,
+        ssl: Optional[SSLContext] = None,
+        timeout: Optional[float] = None,
+    ) -> Tuple[asyncio.Transport, ResponseHandler]:
+        raise NotImplementedError
+
+
+class ProxyConnector(_BaseProxyConnector):
     def __init__(
         self,
-        proxy_type=ProxyType.SOCKS5,
-        host=None,
-        port=None,
-        username=None,
-        password=None,
-        rdns=None,
-        proxy_ssl=None,
-        **kwargs,
-    ):
+        host: str,
+        port: int,
+        proxy_type: ProxyType = ProxyType.SOCKS5,
+        username: Optional[str] = None,
+        password: Optional[str] = None,
+        rdns: Optional[bool] = None,
+        proxy_ssl: Optional[SSLContext] = None,
+        **kwargs: Any,
+    ) -> None:
         kwargs['resolver'] = NoResolver()
         super().__init__(**kwargs)
 
@@ -64,7 +103,13 @@ class ProxyConnector(TCPConnector):
         self._rdns = rdns
         self._proxy_ssl = proxy_ssl
 
-    async def _connect_via_proxy(self, host, port, ssl=None, timeout=None):
+    async def _connect_via_proxy(
+        self,
+        host: str,
+        port: int,
+        ssl: Optional[SSLContext] = None,
+        timeout: Optional[float] = None,
+    ) -> Tuple[asyncio.Transport, ResponseHandler]:
         proxy = Proxy(
             proxy_type=self._proxy_type,
             host=self._proxy_host,
@@ -82,7 +127,7 @@ class ProxyConnector(TCPConnector):
             timeout=timeout,
         )
 
-        transport: BaseTransport = stream.writer.transport
+        transport: asyncio.Transport = stream.writer.transport
         protocol: ResponseHandler = _ResponseHandler(
             loop=self._loop,
             writer=stream.writer,
@@ -93,32 +138,8 @@ class ProxyConnector(TCPConnector):
 
         return transport, protocol
 
-    async def _wrap_create_connection(
-        self,
-        *args,
-        addr_infos,
-        req,
-        timeout,
-        client_error=ClientConnectorError,
-        **kwargs,
-    ):
-        try:
-            host = addr_infos[0][4][0]
-            port = addr_infos[0][4][1]
-        except IndexError:
-            raise ValueError('Invalid arg: `addr_infos`')
-
-        ssl = kwargs.get('ssl')
-
-        return await self._connect_via_proxy(
-            host=host,
-            port=port,
-            ssl=ssl,
-            timeout=timeout.sock_connect,
-        )
-
     @classmethod
-    def from_url(cls, url, **kwargs):
+    def from_url(cls, url: str, **kwargs: Any) -> 'ProxyConnector':
         proxy_type, host, port, username, password = parse_proxy_url(url)
         return cls(
             proxy_type=proxy_type,
@@ -130,23 +151,29 @@ class ProxyConnector(TCPConnector):
         )
 
 
-class ProxyInfo(typing.NamedTuple):
+class ProxyInfo(NamedTuple):
     proxy_type: ProxyType
     host: str
     port: int
-    username: typing.Optional[str] = None
-    password: typing.Optional[str] = None
-    rdns: typing.Optional[bool] = None
+    username: Optional[str] = None
+    password: Optional[str] = None
+    rdns: Optional[bool] = None
 
 
-class ChainProxyConnector(TCPConnector):
+class ChainProxyConnector(_BaseProxyConnector):
     def __init__(self, proxy_infos: Iterable[ProxyInfo], **kwargs):
         kwargs['resolver'] = NoResolver()
         super().__init__(**kwargs)
 
         self._proxy_infos = proxy_infos
 
-    async def _connect_via_proxy(self, host, port, ssl=None, timeout=None):
+    async def _connect_via_proxy(
+        self,
+        host: str,
+        port: int,
+        ssl: Optional[SSLContext] = None,
+        timeout: Optional[float] = None,
+    ) -> Tuple[asyncio.Transport, ResponseHandler]:
         forward = None
         proxy = None
         for info in self._proxy_infos:
@@ -161,6 +188,8 @@ class ChainProxyConnector(TCPConnector):
             )
             forward = proxy
 
+        assert proxy is not None
+
         stream = await proxy.connect(
             dest_host=host,
             dest_port=port,
@@ -168,7 +197,7 @@ class ChainProxyConnector(TCPConnector):
             timeout=timeout,
         )
 
-        transport: BaseTransport = stream.writer.transport
+        transport: asyncio.Transport = stream.writer.transport
         protocol: ResponseHandler = _ResponseHandler(
             loop=self._loop,
             writer=stream.writer,
@@ -179,32 +208,8 @@ class ChainProxyConnector(TCPConnector):
 
         return transport, protocol
 
-    async def _wrap_create_connection(
-        self,
-        *args,
-        addr_infos,
-        req,
-        timeout,
-        client_error=ClientConnectorError,
-        **kwargs,
-    ):
-        try:
-            host = addr_infos[0][4][0]
-            port = addr_infos[0][4][1]
-        except IndexError:
-            raise ValueError('Invalid arg: `addr_infos`')
-
-        ssl = kwargs.get('ssl')
-
-        return await self._connect_via_proxy(
-            host=host,
-            port=port,
-            ssl=ssl,
-            timeout=timeout.sock_connect,
-        )
-
     @classmethod
-    def from_urls(cls, urls: Iterable[str], **kwargs):
+    def from_urls(cls, urls: Iterable[str], **kwargs: Any) -> 'ChainProxyConnector':
         infos = []
         for url in urls:
             proxy_type, host, port, username, password = parse_proxy_url(url)
